@@ -14,5 +14,5 @@ fi
 [[ -d "$SRCDIR" ]] || { echo >&2 "Source not found: $SRCDIR or ${tar_orig}."; exit 255; }
 
 dockername="gs-x86_64-debian-devel"
-docker run --rm -it "${dockername}" true || docker build -t "${dockername}" . || { exit 255; }
+docker run --rm -it "${dockername}" true || (cd "${DEBDIR}" && docker build -t "${dockername}" . ) || { exit 255; }
 docker run --rm  -v "${PKGDIR}:/gsocket-pkg" -v "${SRCDIR}:/gsocket-src" -v "${DEBDIR}:/gsocket-deb" -e VER=$VER -it "${dockername}" /gsocket-deb/build.sh || { exit 255; }
